@@ -1,10 +1,11 @@
 use gitql_ast::environment::Environment;
-use gix::reference::Category;
+use gitql_ast::object::Group;
+use gitql_ast::object::Row;
+use gix::refs::Category;
 use std::collections::HashMap;
 
 use gitql_ast::expression::Expression;
 use gitql_ast::expression::SymbolExpression;
-use gitql_ast::object::GQLObject;
 use gitql_ast::value::Value;
 
 use crate::engine_evaluator::evaluate_expression;
@@ -14,16 +15,16 @@ pub fn select_gql_objects(
     repo: &gix::Repository,
     table: String,
     fields_names: &Vec<String>,
-    fields_values: &Vec<Box<dyn Expression>>,
-    alias_table: &HashMap<String, String>,
-) -> Result<Vec<GQLObject>, String> {
+    titles: &[String],
+    fields_values: &[Box<dyn Expression>],
+) -> Result<Group, String> {
     match table.as_str() {
-        "refs" => select_references(env, repo, fields_names, fields_values, alias_table),
-        "commits" => select_commits(env, repo, fields_names, fields_values, alias_table),
-        "branches" => select_branches(env, repo, fields_names, fields_values, alias_table),
-        "diffs" => select_diffs(env, repo, fields_names, fields_values, alias_table),
-        "tags" => select_tags(env, repo, fields_names, fields_values, alias_table),
-        _ => select_values(env, fields_names, fields_values, alias_table),
+        "refs" => select_references(env, repo, fields_names, titles, fields_values),
+        "commits" => select_commits(env, repo, fields_names, titles, fields_values),
+        "branches" => select_branches(env, repo, fields_names, titles, fields_values),
+        "diffs" => select_diffs(env, repo, fields_names, titles, fields_values),
+        "tags" => select_tags(env, repo, fields_names, titles, fields_values),
+        _ => select_values(env, titles, fields_values),
     }
 }
 
@@ -31,14 +32,15 @@ fn select_references(
     env: &mut Environment,
     repo: &gix::Repository,
     fields_names: &Vec<String>,
-    fields_values: &Vec<Box<dyn Expression>>,
-    alias_table: &HashMap<String, String>,
-) -> Result<Vec<GQLObject>, String> {
+    titles: &[String],
+    fields_values: &[Box<dyn Expression>],
+) -> Result<Group, String> {
     let repo_path = repo.path().to_str().unwrap().to_string();
-    let mut gql_references: Vec<GQLObject> = Vec::new();
+
+    let mut rows: Vec<Row> = vec![];
     let git_references = repo.references();
     if git_references.is_err() {
-        return Ok(gql_references);
+        return Ok(Group { rows });
     }
 
     let references = git_references.ok().unwrap();
@@ -47,7 +49,7 @@ fn select_references(
     let padding = names_len - values_len;
 
     for reference in references.all().unwrap().flatten() {
-        let mut attributes: HashMap<String, Value> = HashMap::with_capacity(fields_names.len());
+        let mut values: Vec<Value> = Vec::with_capacity(fields_names.len());
 
         for index in 0..names_len {
             let field_name = &fields_names[index as usize];
@@ -55,9 +57,8 @@ fn select_references(
             if (index - padding) >= 0 {
                 let value = &fields_values[(index - padding) as usize];
                 if value.as_any().downcast_ref::<SymbolExpression>().is_none() {
-                    let evaluated = evaluate_expression(env, value, &attributes)?;
-                    let column_name = get_column_name(alias_table, field_name);
-                    attributes.insert(column_name, evaluated);
+                    let evaluated = evaluate_expression(env, value, titles, &values)?;
+                    values.push(evaluated);
                     continue;
                 }
             }
@@ -69,62 +70,60 @@ fn select_references(
                     .map(|(_, sn)| sn)
                     .unwrap_or("".into())
                     .to_string();
-                let column_name = get_column_name(alias_table, "name");
-                attributes.insert(column_name, Value::Text(name));
+                values.push(Value::Text(name));
                 continue;
             }
 
             if field_name == "full_name" {
                 let full_name = reference.name().as_bstr().to_string();
-                let column_name = get_column_name(alias_table, "full_name");
-                attributes.insert(column_name, Value::Text(full_name));
+                values.push(Value::Text(full_name));
                 continue;
             }
 
             if field_name == "type" {
-                let column_name = get_column_name(alias_table, "type");
                 let category = reference.name().category();
                 if category.map_or(false, |cat| cat == Category::LocalBranch) {
-                    attributes.insert(column_name, Value::Text("branch".to_owned()));
+                    values.push(Value::Text("branch".to_owned()));
                 } else if category.map_or(false, |cat| cat == Category::RemoteBranch) {
-                    attributes.insert(column_name, Value::Text("remote".to_owned()));
+                    values.push(Value::Text("remote".to_owned()));
                 } else if category.map_or(false, |cat| cat == Category::Tag) {
-                    attributes.insert(column_name, Value::Text("tag".to_owned()));
+                    values.push(Value::Text("tag".to_owned()));
                 } else if category.map_or(false, |cat| cat == Category::Note) {
-                    attributes.insert(column_name, Value::Text("note".to_owned()));
+                    values.push(Value::Text("note".to_owned()));
                 } else {
-                    attributes.insert(column_name, Value::Text("other".to_owned()));
+                    values.push(Value::Text("other".to_owned()));
                 }
                 continue;
             }
 
             if field_name == "repo" {
-                let column_name = get_column_name(alias_table, "repo");
-                attributes.insert(column_name, Value::Text(repo_path.to_string()));
+                values.push(Value::Text(repo_path.to_string()));
                 continue;
             }
+
+            values.push(Value::Null);
         }
 
-        let gql_reference = GQLObject { attributes };
-        gql_references.push(gql_reference);
+        let row = Row { values };
+        rows.push(row);
     }
 
-    Ok(gql_references)
+    Ok(Group { rows })
 }
 
 fn select_commits(
     env: &mut Environment,
     repo: &gix::Repository,
     fields_names: &Vec<String>,
-    fields_values: &Vec<Box<dyn Expression>>,
-    alias_table: &HashMap<String, String>,
-) -> Result<Vec<GQLObject>, String> {
+    titles: &[String],
+    fields_values: &[Box<dyn Expression>],
+) -> Result<Group, String> {
     let repo_path = repo.path().to_str().unwrap().to_string();
 
-    let mut commits: Vec<GQLObject> = Vec::new();
+    let mut rows: Vec<Row> = vec![];
     let head_id = repo.head_id();
     if head_id.is_err() {
-        return Ok(commits);
+        return Ok(Group { rows });
     }
 
     let revwalk = head_id.unwrap().ancestors().all().unwrap();
@@ -138,7 +137,7 @@ fn select_commits(
         let commit = repo.find_object(commit_info.id).unwrap().into_commit();
         let commit = commit.decode().unwrap();
 
-        let mut attributes: HashMap<String, Value> = HashMap::with_capacity(fields_names.len());
+        let mut values: Vec<Value> = Vec::with_capacity(fields_names.len());
 
         for index in 0..names_len {
             let field_name = &fields_names[index as usize];
@@ -146,85 +145,173 @@ fn select_commits(
             if (index - padding) >= 0 {
                 let value = &fields_values[(index - padding) as usize];
                 if value.as_any().downcast_ref::<SymbolExpression>().is_none() {
-                    let evaluated = evaluate_expression(env, value, &attributes)?;
-                    let column_name = get_column_name(alias_table, field_name);
-                    attributes.insert(column_name, evaluated);
+                    let evaluated = evaluate_expression(env, value, titles, &values)?;
+                    values.push(evaluated);
                     continue;
                 }
             }
 
             if field_name == "commit_id" {
                 let commit_id = Value::Text(commit_info.id.to_string());
-                let column_name = get_column_name(alias_table, "commit_id");
-                attributes.insert(column_name, commit_id);
+                values.push(commit_id);
                 continue;
             }
 
             if field_name == "name" {
                 let name = commit.author().name.to_string();
-                let column_name = get_column_name(alias_table, "name");
-                attributes.insert(column_name, Value::Text(name));
+                values.push(Value::Text(name));
                 continue;
             }
 
             if field_name == "email" {
                 let email = commit.author().email.to_string();
-                let column_name = get_column_name(alias_table, "email");
-                attributes.insert(column_name, Value::Text(email));
+                values.push(Value::Text(email));
                 continue;
             }
 
             if field_name == "title" {
                 let summary = Value::Text(commit.message().summary().to_string());
-                let column_name = get_column_name(alias_table, "title");
-                attributes.insert(column_name, summary);
+                values.push(summary);
                 continue;
             }
 
             if field_name == "message" {
                 let message = Value::Text(commit.message.to_string());
-                let column_name = get_column_name(alias_table, "message");
-                attributes.insert(column_name, message);
+                values.push(message);
                 continue;
             }
 
             if field_name == "datetime" {
-                let column_name = get_column_name(alias_table, "datetime");
                 let time_stamp = commit_info
                     .commit_time
                     .unwrap_or_else(|| commit.time().seconds);
-                attributes.insert(column_name, Value::DateTime(time_stamp));
+                values.push(Value::DateTime(time_stamp));
                 continue;
             }
 
             if field_name == "repo" {
-                let column_name = get_column_name(alias_table, "repo");
-                attributes.insert(column_name, Value::Text(repo_path.to_string()));
+                values.push(Value::Text(repo_path.to_string()));
                 continue;
             }
+
+            values.push(Value::Null);
         }
 
-        let gql_commit = GQLObject { attributes };
-        commits.push(gql_commit);
+        let row = Row { values };
+        rows.push(row);
     }
 
-    Ok(commits)
+    Ok(Group { rows })
+}
+
+fn select_branches(
+    env: &mut Environment,
+    repo: &gix::Repository,
+    fields_names: &Vec<String>,
+    titles: &[String],
+    fields_values: &[Box<dyn Expression>],
+) -> Result<Group, String> {
+    let mut rows: Vec<Row> = vec![];
+
+    let repo_path = repo.path().to_str().unwrap().to_string();
+    let platform = repo.references().unwrap();
+    let local_branches = platform.local_branches().unwrap();
+    let remote_branches = platform.remote_branches().unwrap();
+    let local_and_remote_branches = local_branches.chain(remote_branches);
+    let head_ref_result = repo.head_ref();
+    if head_ref_result.is_err() {
+        return Ok(Group { rows });
+    }
+
+    let head_ref_option = head_ref_result.unwrap();
+    if head_ref_option.is_none() {
+        return Ok(Group { rows });
+    }
+
+    let head_ref = head_ref_option.unwrap();
+
+    let names_len = fields_names.len() as i64;
+    let values_len = fields_values.len() as i64;
+    let padding = names_len - values_len;
+
+    for branch in local_and_remote_branches.flatten() {
+        let mut values: Vec<Value> = Vec::with_capacity(fields_names.len());
+
+        for index in 0..names_len {
+            let field_name = &fields_names[index as usize];
+
+            if (index - padding) >= 0 {
+                let value = &fields_values[(index - padding) as usize];
+                if value.as_any().downcast_ref::<SymbolExpression>().is_none() {
+                    let evaluated = evaluate_expression(env, value, titles, &values)?;
+                    values.push(evaluated);
+                    continue;
+                }
+            }
+
+            if field_name == "name" {
+                let branch_name = branch.name().as_bstr().to_string();
+                values.push(Value::Text(branch_name));
+                continue;
+            }
+
+            if field_name == "commit_count" {
+                let commit_count = if let Some(id) = branch.try_id() {
+                    if let Ok(revwalk) = id.ancestors().all() {
+                        revwalk.count() as i64
+                    } else {
+                        -1
+                    }
+                } else {
+                    -1
+                };
+                values.push(Value::Integer(commit_count));
+                continue;
+            }
+
+            if field_name == "is_head" {
+                values.push(Value::Boolean(branch.inner == head_ref.inner));
+                continue;
+            }
+
+            if field_name == "is_remote" {
+                let is_remote = branch
+                    .name()
+                    .category()
+                    .map_or(false, |cat| cat == Category::RemoteBranch);
+                values.push(Value::Boolean(is_remote));
+                continue;
+            }
+
+            if field_name == "repo" {
+                values.push(Value::Text(repo_path.to_string()));
+                continue;
+            }
+
+            values.push(Value::Null);
+        }
+
+        let row = Row { values };
+        rows.push(row);
+    }
+
+    Ok(Group { rows })
 }
 
 fn select_diffs(
     env: &mut Environment,
     repo: &gix::Repository,
     fields_names: &Vec<String>,
-    fields_values: &Vec<Box<dyn Expression>>,
-    alias_table: &HashMap<String, String>,
-) -> Result<Vec<GQLObject>, String> {
+    titles: &[String],
+    fields_values: &[Box<dyn Expression>],
+) -> Result<Group, String> {
     let repo = {
         let mut repo = repo.clone();
         repo.object_cache_size_if_unset(4 * 1024 * 1024);
         repo
     };
 
-    let mut diffs: Vec<GQLObject> = Vec::new();
+    let mut rows: Vec<Row> = vec![];
     let revwalk = repo.head_id().unwrap().ancestors().all().unwrap();
     let repo_path = repo.path().to_str().unwrap().to_string();
 
@@ -240,7 +327,8 @@ fn select_diffs(
     for commit_info in revwalk {
         let commit_info = commit_info.unwrap();
         let commit = commit_info.id().object().unwrap().into_commit();
-        let mut attributes: HashMap<String, Value> = HashMap::with_capacity(fields_names.len());
+
+        let mut values: Vec<Value> = Vec::with_capacity(fields_names.len());
 
         for index in 0..names_len {
             let field_name = &fields_names[index as usize];
@@ -248,36 +336,31 @@ fn select_diffs(
             if (index - padding) >= 0 {
                 let value = &fields_values[(index - padding) as usize];
                 if value.as_any().downcast_ref::<SymbolExpression>().is_none() {
-                    let evaluated = evaluate_expression(env, value, &attributes)?;
-                    let column_name = get_column_name(alias_table, field_name);
-                    attributes.insert(column_name, evaluated);
+                    let evaluated = evaluate_expression(env, value, titles, &values)?;
+                    values.push(evaluated);
                     continue;
                 }
             }
 
             if field_name == "commit_id" {
-                let column_name = get_column_name(alias_table, "commit_id");
-                attributes.insert(column_name, Value::Text(commit_info.id.to_string()));
+                values.push(Value::Text(commit_info.id.to_string()));
                 continue;
             }
 
             if field_name == "name" {
                 let name = commit.author().unwrap().name.to_string();
-                let column_name = get_column_name(alias_table, "name");
-                attributes.insert(column_name, Value::Text(name));
+                values.push(Value::Text(name));
                 continue;
             }
 
             if field_name == "email" {
                 let email = commit.author().unwrap().email.to_string();
-                let column_name = get_column_name(alias_table, "email");
-                attributes.insert(column_name, Value::Text(email));
+                values.push(Value::Text(email));
                 continue;
             }
 
             if field_name == "repo" {
-                let column_name = get_column_name(alias_table, "repo");
-                attributes.insert(column_name, Value::Text(repo_path.to_string()));
+                values.push(Value::Text(repo_path.to_string()));
                 continue;
             }
 
@@ -322,136 +405,38 @@ fn select_diffs(
                     .unwrap();
 
                 if field_name == "insertions" {
-                    let column_name = get_column_name(alias_table, "insertions");
-                    attributes.insert(column_name, Value::Integer(insertions as i64));
+                    values.push(Value::Integer(insertions as i64));
                     continue;
                 }
 
                 if field_name == "deletions" {
-                    let column_name = get_column_name(alias_table, "deletions");
-                    attributes.insert(column_name, Value::Integer(deletions as i64));
+                    values.push(Value::Integer(deletions as i64));
                     continue;
                 }
 
                 if field_name == "files_changed" {
-                    let column_name = get_column_name(alias_table, "files_changed");
-                    attributes.insert(column_name, Value::Integer(files_changed as i64));
-                    continue;
-                }
-            }
-        }
-
-        let gql_diff = GQLObject { attributes };
-        diffs.push(gql_diff);
-    }
-
-    Ok(diffs)
-}
-
-fn select_branches(
-    env: &mut Environment,
-    repo: &gix::Repository,
-    fields_names: &Vec<String>,
-    fields_values: &Vec<Box<dyn Expression>>,
-    alias_table: &HashMap<String, String>,
-) -> Result<Vec<GQLObject>, String> {
-    let mut branches: Vec<GQLObject> = Vec::new();
-    let repo_path = repo.path().to_str().unwrap().to_string();
-    let platform = repo.references().unwrap();
-    let local_branches = platform.local_branches().unwrap();
-    let remote_branches = platform.remote_branches().unwrap();
-    let local_and_remote_branches = local_branches.chain(remote_branches);
-    let head_ref_result = repo.head_ref();
-    if head_ref_result.is_err() {
-        return Ok(branches);
-    }
-
-    let head_ref_option = head_ref_result.unwrap();
-    if head_ref_option.is_none() {
-        return Ok(branches);
-    }
-
-    let head_ref = head_ref_option.unwrap();
-
-    let names_len = fields_names.len() as i64;
-    let values_len = fields_values.len() as i64;
-    let padding = names_len - values_len;
-
-    for branch in local_and_remote_branches.flatten() {
-        let mut attributes: HashMap<String, Value> = HashMap::with_capacity(fields_names.len());
-        for index in 0..names_len {
-            let field_name = &fields_names[index as usize];
-
-            if (index - padding) >= 0 {
-                let value = &fields_values[(index - padding) as usize];
-                if value.as_any().downcast_ref::<SymbolExpression>().is_none() {
-                    let evaluated = evaluate_expression(env, value, &attributes)?;
-                    let column_name = get_column_name(alias_table, field_name);
-                    attributes.insert(column_name, evaluated);
+                    values.push(Value::Integer(files_changed as i64));
                     continue;
                 }
             }
 
-            if field_name == "name" {
-                let branch_name = branch.name().as_bstr().to_string();
-                let column_name = get_column_name(alias_table, "name");
-                attributes.insert(column_name, Value::Text(branch_name));
-                continue;
-            }
-
-            if field_name == "commit_count" {
-                let commit_count = if let Some(id) = branch.try_id() {
-                    if let Ok(revwalk) = id.ancestors().all() {
-                        revwalk.count() as i64
-                    } else {
-                        -1
-                    }
-                } else {
-                    -1
-                };
-                let column_name = get_column_name(alias_table, "commit_count");
-                attributes.insert(column_name, Value::Integer(commit_count));
-                continue;
-            }
-
-            if field_name == "is_head" {
-                let column_name = get_column_name(alias_table, "is_head");
-                attributes.insert(column_name, Value::Boolean(branch.inner == head_ref.inner));
-                continue;
-            }
-
-            if field_name == "is_remote" {
-                let column_name = get_column_name(alias_table, "is_remote");
-                let is_remote = branch
-                    .name()
-                    .category()
-                    .map_or(false, |cat| cat == Category::RemoteBranch);
-                attributes.insert(column_name, Value::Boolean(is_remote));
-                continue;
-            }
-
-            if field_name == "repo" {
-                let column_name = get_column_name(alias_table, "repo");
-                attributes.insert(column_name, Value::Text(repo_path.to_string()));
-                continue;
-            }
+            values.push(Value::Null);
         }
 
-        let gql_branch = GQLObject { attributes };
-        branches.push(gql_branch);
+        let row = Row { values };
+        rows.push(row);
     }
 
-    Ok(branches)
+    Ok(Group { rows })
 }
 
 fn select_tags(
     env: &mut Environment,
     repo: &gix::Repository,
     fields_names: &Vec<String>,
-    fields_values: &Vec<Box<dyn Expression>>,
-    alias_table: &HashMap<String, String>,
-) -> Result<Vec<GQLObject>, String> {
-    let mut tags: Vec<GQLObject> = Vec::new();
+    titles: &[String],
+    fields_values: &[Box<dyn Expression>],
+) -> Result<Group, String> {
     let platform = repo.references().unwrap();
     let tag_names = platform.tags().unwrap();
     let repo_path = repo.path().to_str().unwrap().to_string();
@@ -460,8 +445,10 @@ fn select_tags(
     let values_len = fields_values.len() as i64;
     let padding = names_len - values_len;
 
+    let mut rows: Vec<Row> = vec![];
+
     for tag_ref in tag_names.flatten() {
-        let mut attributes: HashMap<String, Value> = HashMap::with_capacity(fields_names.len());
+        let mut values: Vec<Value> = Vec::with_capacity(fields_names.len());
 
         for index in 0..names_len {
             let field_name = &fields_names[index as usize];
@@ -469,55 +456,51 @@ fn select_tags(
                 let value = &fields_values[(index - padding) as usize];
 
                 if value.as_any().downcast_ref::<SymbolExpression>().is_none() {
-                    let evaluated = evaluate_expression(env, value, &attributes)?;
-                    let column_name = get_column_name(alias_table, field_name);
-                    attributes.insert(column_name, evaluated);
+                    let evaluated = evaluate_expression(env, value, titles, &values)?;
+                    values.push(evaluated);
                     continue;
                 }
             }
 
             if field_name == "name" {
-                let column_name = get_column_name(alias_table, "name");
                 let tag_name = tag_ref
                     .name()
                     .category_and_short_name()
                     .map_or_else(String::default, |(_, short_name)| short_name.to_string());
-                attributes.insert(column_name, Value::Text(tag_name.to_string()));
+                values.push(Value::Text(tag_name.to_string()));
                 continue;
             }
 
             if field_name == "repo" {
-                let column_name = get_column_name(alias_table, "repo");
-                attributes.insert(column_name, Value::Text(repo_path.to_string()));
+                values.push(Value::Text(repo_path.to_string()));
                 continue;
             }
+
+            values.push(Value::Null);
         }
 
-        let gql_tag = GQLObject { attributes };
-        tags.push(gql_tag);
+        let row = Row { values };
+        rows.push(row);
     }
 
-    Ok(tags)
+    Ok(Group { rows })
 }
 
 fn select_values(
     env: &mut Environment,
-    fields_names: &[String],
-    fields_values: &Vec<Box<dyn Expression>>,
-    alias_table: &HashMap<String, String>,
-) -> Result<Vec<GQLObject>, String> {
-    let len = fields_values.len();
-    let mut attributes: HashMap<String, Value> = HashMap::with_capacity(len);
+    titles: &[String],
+    fields_values: &[Box<dyn Expression>],
+) -> Result<Group, String> {
+    let mut group = Group { rows: vec![] };
+    let mut values = Vec::with_capacity(fields_values.len());
 
-    for index in 0..len {
-        let field_name = &fields_names[index];
-        let value = &fields_values[index];
-        let evaluated = evaluate_expression(env, value, &attributes)?;
-        let column_name = get_column_name(alias_table, field_name);
-        attributes.insert(column_name, evaluated);
+    for value in fields_values.iter() {
+        let evaluated = evaluate_expression(env, value, titles, &values)?;
+        values.push(evaluated);
     }
 
-    Ok(vec![GQLObject { attributes }])
+    group.rows.push(Row { values });
+    Ok(group)
 }
 
 #[inline(always)]
